----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    11:06:48 12/18/2009 
-- Design Name: 
-- Module Name:    control_unit - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity control_unit is
	PORT(
		CLK_in		: IN std_logic;		
		RST_in		: IN std_logic;
		
		-- RS232
		RXD_in		: IN std_logic;
				
		-- LCD
		LCD_E_out	: out	STD_LOGIC;
		LCD_RS_out	: out	STD_LOGIC;
		LCD_RW_out	: out	STD_LOGIC;
		SF_D_inout	: inout	STD_LOGIC_VECTOR(11 downto 8)
		);

end control_unit;

architecture Behavioral of control_unit is
	
	-- COMPONENTS
	COMPONENT pulse_gen
	PORT(
		CLK_in : IN std_logic;
		RST_in : IN std_logic;          
		Pulse_out : OUT std_logic
		);
	END COMPONENT;
	
	COMPONENT shift_reg
	PORT(
		CLK_in : IN std_logic;
		RST_in : IN std_logic;
		SE_in : IN std_logic;
		VALUE_in : IN std_logic;          
		VALUE_out : OUT std_logic_vector(7 downto 0)
		);
	END COMPONENT;
	
	COMPONENT automaton
	PORT(
		CLK_in : IN std_logic;
		RST_in : IN std_logic;
		DATA_in : IN std_logic_vector(7 downto 0);
		DATA_RDY_in : IN std_logic;    
		SF_D_inout : INOUT std_logic_vector(11 downto 8);      
		LCD_E_out : OUT std_logic;
		LCD_RS_out : OUT std_logic;
		LCD_RW_out : OUT std_logic
		);
	END COMPONENT;

	-- INTERNAL SIGNALS
		
	-- AUTOMATON STATES
	type states is (idle, start, stop, bit0, bit1, bit2, bit3, bit4, bit5, bit6, bit7);
	signal currSt, nextSt : states;
	signal RS232_CLK : std_logic;
	signal PG_RST, nextPG_RST : std_logic; -- Pulse Gen Reset
	signal SE, nextSE : std_logic; -- Shift Enable
	signal DataRDY, nextDataRDY: std_logic; -- data ready on RxD
	signal RxD : std_logic;
	signal data : std_logic_vector(7 downto 0);

begin

	-- IO ASSIGNMENT
	RxD <= RxD_in;

	-- COMPONENT INSTANTIATION
	Inst_pulse_gen: pulse_gen PORT MAP(
		CLK_in => CLK_in,
		RST_in => PG_RST,
		Pulse_out => RS232_CLK
	);
	
	
	Inst_shift_reg: shift_reg PORT MAP(
		CLK_in => CLK_in,
		RST_in => RST_in,
		SE_in => SE,
		VALUE_in => RxD,
		VALUE_out => data
	);
	
	-- lcd
	lcd_automaton: automaton PORT MAP(
		CLK_in => CLK_in,
		RST_in => RST_in,
		DATA_in => data,
		DATA_RDY_in => DataRDY,
		LCD_E_out => LCD_E_out,
		LCD_RS_out => LCD_RS_out,
		LCD_RW_out => LCD_RW_out,
		SF_D_inout => SF_D_inout 
	);
	
	
	-- AUTOMATON
	process (CLK_in)
	begin
		if (rising_edge(CLK_in)) then
			if (RST_in = '1') then
				currSt	<= idle;
				PG_RST	<= '0';
				SE			<= '0';
				DataRDY	<= '0';
				
			else
				currSt	<= nextSt;
				PG_RST	<= nextPG_RST;				
				SE			<= nextSE;
				DataRDY	<= nextDataRDY;
			end if;
		end if;	
	end process;

	-- MEALY
	transition_func: process (currSt, RxD, RS232_CLK)
	begin
		-- default values;
		nextSt <= currSt;
		
		case currSt is
		-- IDLE
		when idle =>
			if (RxD = '0') then
				nextSt	<= start;
			else
				nextSt	<= idle;
			end if;
			
		-- START
		when start =>
			if (RS232_CLK = '1') then			
				if (RxD = '0') then
					nextSt	<= bit0;
				else
					nextSt	<= idle;
				end if;
			end if;
		-- BIT0
		when bit0 =>
			if (RS232_CLK = '1') then
				nextSt 	<= bit1;
			end if;
		-- BIT1
		when bit1 =>
			if (RS232_CLK = '1') then
				nextSt 	<= bit2;
			end if;
		-- BIT2
		when bit2 =>
			if (RS232_CLK = '1') then
				nextSt 	<= bit3;
			end if;
		-- BIT3
		when bit3 =>
			if (RS232_CLK = '1') then
				nextSt 	<= bit4;
			end if;
		-- BIT4
		when bit4 =>
			if (RS232_CLK = '1') then
				nextSt 	<= bit5;
			end if;
		-- BIT5
		when bit5 =>
			if (RS232_CLK = '1') then
				nextSt 	<= bit6;
			end if;
		-- BIT6
		when bit6 =>
			if (RS232_CLK = '1') then
				nextSt 	<= bit7;
			end if;
		-- BIT7
		when bit7 =>
			if (RS232_CLK = '1') then
				nextSt 	<= stop;
			end if;
		-- STOP
		when stop =>
			if (RS232_CLK = '1') then
				nextSt	<= idle;
			end if;			
		end case;
	end process;
	
	output_func:process (currSt, RxD, RS232_CLK)
	begin
		-- default values;
		nextPG_RST	<= '0';
		nextSE		<= '0';
		nextDataRDY <= '0';
		
		case currSt is
		-- IDLE
		when idle =>
			if (RxD = '0') then
				nextPG_RST	<= '1';
			end if;
		-- BIT0
		when bit0 =>
			if (RS232_CLK = '1') then
				nextSE 	<= '1';
			end if;
		-- BIT1
		when bit1 =>
			if (RS232_CLK = '1') then
				nextSE 	<= '1';
			end if;
		-- BIT2
		when bit2 =>
			if (RS232_CLK = '1') then
				nextSE 	<= '1';
			end if;
		-- BIT3
		when bit3 =>
			if (RS232_CLK = '1') then
				nextSE 	<= '1';
			end if;
		-- BIT4
		when bit4 =>
			if (RS232_CLK = '1') then
				nextSE 	<= '1';
			end if;
		-- BIT5
		when bit5 =>
			if (RS232_CLK = '1') then
				nextSE 	<= '1';
			end if;
		-- BIT6
		when bit6 =>
			if (RS232_CLK = '1') then
				nextSE 	<= '1';
			end if;
		-- BIT7
		when bit7 =>
			if (RS232_CLK = '1') then
				nextSE 	<= '1';
			end if;
		-- STOP
		when stop =>
			if (RS232_CLK = '1') then
				nextDataRDY <= '1';
			end if;	
		when others =>
		end case;		
	end process;
	
	


end Behavioral;

